/**
 * synopsis: Generate fbpanel menu file from a JWM configuration file
 * purpose: The purpose of this utility is to keep the fbpanel4 synchronized with the 
 *          JWM menu
 * usage: jwm2fbpanel-menu <.jwmrc>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 *
 *To compile this file using gcc you can type
 *gcc `xml2-config --cflags --libs` -o jwm2fbpanel-menu jwm2fbpanel-menu.c
 */

#include <stdio.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#ifndef LIBXML_TREE_ENABLED
int main(void) {
    fprintf(stderr, "Tree support not compiled in\n");
    exit(1);
}

#else

/**** Function declaration */
static void cleanup_input(char * jwmFileName);
static void process_nodes(xmlNodePtr a_node);
static void add_item(xmlNodePtr curNode);


/**** Main function */
int
main(int argc, char **argv)
{
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;

    if (((argc < 2) || (argc > 3))
     || (strcmp(argv[1], "-h")==0)
     || (strcmp(argv[1], "-help")==0)
     || (strcmp(argv[1], "--help")==0))
    {
      printf ("Translator from a JWM configuration file to an fbpanel menu plugin configuration section.\n");
      printf ("It sends the output to stdout.\n");
      printf ("Usage:\n");
      printf ("  jwm2-menu .jwmrc\n");
      printf ("\n");
      printf ("  .jwmrc  : JWM configuration file\n\n");
      printf ("For example:\n");
      printf ("  jwm2fbpanel-menu \"/home/user/.jwmrc\" > menu.config\n\n");
      return(1);
    }

    LIBXML_TEST_VERSION
    
    cleanup_input(argv[1]);

    /*parse the file and get the DOM */
    doc = xmlReadFile("/tmp/jwmMenu", NULL, 0);

    if (doc == NULL) 
    {
        printf("error: could not parse file %s\n", argv[1]);
    }
    else
    {

      /* Initialize the destination file */
      printf("Plugin { \n");
      printf("    type = menu\n");

      /*Get the root element node */
      root_element = xmlDocGetRootElement(doc);

      process_nodes(root_element);

      printf("} \n");
    }

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    return 0;
}

/**
 * Cleans up the incoming jwmFilename: Extracts only the RootMenu node without comments
 * @jwmFilename: the input jwm configuration file
 */
void 
cleanup_input(char * jwmFileName)
{
    char *awkCommand1 = "awk '/<RootMenu/,/RootMenu>/' ";
    char *awkCommand2 = " | awk '/<!--/,/-->/ { next }; { print $0 }' > /tmp/jwmMenu";

    int  totLen = strlen(awkCommand1) + strlen(jwmFileName) + strlen(awkCommand2) + 1;
    
    char *awkCommand = (char *)calloc(totLen, sizeof(char));

    strcpy(awkCommand, awkCommand1);
    strcat(awkCommand, jwmFileName);
    strcat(awkCommand, awkCommand2);

    system(awkCommand);
    
    free(awkCommand);
}

/**
 * process a node and all it's sibling and children nodes
 * @a_node: the initial xml node to consider.
 * @newParentNode: The current parent node in the new document.
 *
 * If the node belongs to the RootMenu element, it 
 * translates from the jwm menu nodes and attributes to the 
 * corresponding xfce menu nodes and attributes
 * and writes them to the destination document.
 */

static void
process_nodes(xmlNodePtr a_node)
{
  xmlNodePtr curNode = NULL;

  for (curNode = a_node; curNode; curNode = curNode->next) 
  {
    if (curNode->type == XML_ELEMENT_NODE) 
    {
        if (strcmp(curNode->name, "RootMenu")==0)
        {
          printf("    config { \n");
          printf("        image = %s \n", xmlGetProp(curNode, "icon"));
          process_nodes(curNode->children);
          printf("    } \n");
        }
        if (strcmp(curNode->name, "Menu")==0)
        {
          printf("        menu { \n");
          printf("            name  = %s \n", xmlGetProp(curNode, "label"));
          printf("            image = %s \n", xmlGetProp(curNode, "icon"));
          process_nodes(curNode->children);
          printf("        } \n");
        }
        else if (strcmp(curNode->name, "Program")==0)
        {
          add_item(curNode);
        }
        else if (strcmp(curNode->name, "Separator")==0)
        {
          printf("        separator { \n");   
          printf("        } \n");
        }
    }

  }
}

void 
add_item(xmlNodePtr curNode)
{
  xmlChar * command = curNode->children->content;
  xmlChar * command2 = NULL;
  if ((strlen(command) > 5) && (strncmp (command, "exec ", 5) == 0))
  {
    command2 = malloc (strlen (command) - 4);
    strcpy (command2, command + 5);
    command = command2;
  }

  printf("            item { \n");
  printf("                image = %s \n", xmlGetProp(curNode, "icon"));
  printf("                name = %s \n", xmlGetProp(curNode, "label"));
  printf("                action = %s \n", command );
  printf("            } \n");

  if (command == command2)
  {
    xmlFree(command2);
  }
}


#endif


